@@ -61,6 +61,7 @@ import org.aspectj.bridge.ILifecycleAware;
 import org.aspectj.bridge.IMessage;
 import org.aspectj.bridge.IMessageHandler;
 import org.aspectj.bridge.IProgressListener;
+import org.aspectj.bridge.ISourceLocation;
 import org.aspectj.bridge.Message;
 import org.aspectj.bridge.MessageUtil;
 import org.aspectj.bridge.SourceLocation;
@@ -78,7 +79,9 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.IProblemFactory;
 import org.aspectj.org.eclipse.jdt.internal.compiler.batch.ClasspathLocation;
 import org.aspectj.org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
 import org.aspectj.org.eclipse.jdt.internal.compiler.batch.FileSystem;
+import org.aspectj.org.eclipse.jdt.internal.compiler.classfmt.ClassFileConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
+import org.aspectj.org.eclipse.jdt.internal.compiler.env.IModule;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.INameEnvironment;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.parser.Parser;
@@ -106,6 +109,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	static final boolean FAIL_IF_RUNTIME_NOT_FOUND = false;
 
 	private static final FileFilter binarySourceFilter = new FileFilter() {
+		@Override
 		public boolean accept(File f) {
 			return f.getName().endsWith(".class");
 		}
@@ -508,6 +512,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		}
 		// Get a list of all files (i.e. everything that isnt a directory)
 		File[] files = FileUtil.listFiles(dir, new FileFilter() {
+			@Override
 			public boolean accept(File f) {
 				boolean accept = !(f.isDirectory() || f.getName().endsWith(".class"));
 				return accept;
@@ -865,8 +870,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 			bcelWorld.getLint().setFromProperties(buildConfig.getLintSpecFile());
 		}
 
-		for (Iterator i = buildConfig.getAspectpath().iterator(); i.hasNext();) {
-			File f = (File) i.next();
+		for (File f: buildConfig.getAspectpath()) {
 			if (!f.exists()) {
 				IMessage message = new Message("invalid aspectpath entry: " + f.getName(), null, true);
 				handler.handleMessage(message);
@@ -943,7 +947,19 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		// a classpathDirectory object that will attempt to look for source when it can't find binary.
 		// int[] classpathModes = new int[classpaths.length];
 		// for (int i =0 ;i<classpaths.length;i++) classpathModes[i]=ClasspathDirectory.BINARY;
-		return new FileSystem(classpaths, filenames, defaultEncoding, ClasspathLocation.BINARY);
+		
+		FileSystem nameEnvironment = null;
+		// TODO J9 The compiler likes to work in terms of checked classpath objects - these will be different
+		// depending on where the code came from (classpath, modulepath). If working with just the raw
+		// 'classpaths' object it isn't recording where the code came from. This will be an issue later for
+		// weaving, the distinction will need to be maintained for proper 'module aware/respecting' weaving.
+		if (buildConfig.getCheckedClasspaths() == null) {
+			nameEnvironment = new FileSystem(classpaths, filenames, defaultEncoding, ClasspathLocation.BINARY);
+		} else {
+			nameEnvironment = new FileSystem(buildConfig.getCheckedClasspaths(), filenames, false, null);
+		}
+		nameEnvironment.module = buildConfig.getModuleDesc();
+		return nameEnvironment;
 	}
 
 	public IProblemFactory getProblemFactory() {
@@ -962,9 +978,35 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		if ("".equals(defaultEncoding)) {//$NON-NLS-1$
 			defaultEncoding = null;
 		}
-
+		CompilationUnit moduleCU = null;
+		
+		// TODO building with multiple module-infos?
+		int moduleIndex = -1;
+		IModule moduleDesc = buildConfig.getModuleDesc();
+		String moduleName = moduleDesc == null? null: new String(moduleDesc.name());
+		for (int i=0;i<fileCount;i++) {
+			if (filenames[i].endsWith("module-info.java")) {
+				moduleIndex = i;
+				moduleCU = new CompilationUnit(null, filenames[i], defaultEncoding, null, false, moduleName);
+			}
+		}
+		
 		for (int i = 0; i < fileCount; i++) {
-			units[i] = new CompilationUnit(null, filenames[i], defaultEncoding);
+//			units[i] = new CompilationUnit(null, filenames[i], defaultEncoding);
+			if (i == moduleIndex) {
+				units[i] = moduleCU;
+			} else {
+				units[i] = new CompilationUnit(null, filenames[i], defaultEncoding, null, false, moduleName);
+				// With Java 10 changes the modulebinding is fetched from the rootenvironment 
+				// this.moduleBinding = rootEnvironment.getModule(this.module);
+				// rather than using the moduleCU:
+				// if (this.modCU != null)
+				// 	return this.moduleBinding = this.modCU.module(rootEnvironment);
+//				units[i].setModule(moduleCU);
+			}
+//			new CompilationUnit(null, fileName, encoding, this.destinationPaths[i],
+//					shouldIgnoreOptionalProblems(this.ignoreOptionalProblemsFromFolders, fileName.toCharArray()), 
+//					this.modNames[i]);
 		}
 		return units;
 	}
@@ -1013,7 +1055,9 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 			for (int i = 0; i < cps.size(); i++) {
 				classpaths[i] = cps.get(i);
 			}
-			environment = new StatefulNameEnvironment(getLibraryAccess(classpaths, filenames), state.getClassNameToFileMap(), state);
+			FileSystem fileSystem = getLibraryAccess(classpaths, filenames);
+			environment = new StatefulNameEnvironment(fileSystem, state.getClassNameToFileMap(), state);
+			state.setFileSystem(fileSystem);
 			state.setNameEnvironment(environment);
 		} else {
 			((StatefulNameEnvironment) environment).update(state.getClassNameToFileMap(), state.deltaAddedClasses);
@@ -1021,7 +1065,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		}
 
 		org.aspectj.ajdt.internal.compiler.CompilerAdapter.setCompilerAdapterFactory(this);
-		final Map<?, ?> settings = buildConfig.getOptions().getMap();
+		final Map<String, String> settings = buildConfig.getOptions().getMap();
 		final BuildArgParser bMain = buildConfig.getBuildArgParser();
 
 		final org.aspectj.org.eclipse.jdt.internal.compiler.Compiler compiler = new org.aspectj.org.eclipse.jdt.internal.compiler.Compiler(
@@ -1031,6 +1075,33 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		bMain.batchCompiler = compiler;
 		bMain.initializeAnnotationProcessorManager();
 		compiler.options.produceReferenceInfo = true; // TODO turn off when not needed
+		
+
+		if (bMain.compilerOptions.complianceLevel >= ClassFileConstants.JDK1_6
+				&& bMain.compilerOptions.processAnnotations) {
+			// need this too?
+//			if (bMain.checkVMVersion(ClassFileConstants.JDK1_6)) {
+//				initializeAnnotationProcessorManager();
+//				if (this.classNames != null) {
+//					this.batchCompiler.setBinaryTypes(processClassNames(this.batchCompiler.lookupEnvironment));
+//				}
+//			} else {
+//				// report a warning
+//				this.logger.logIncorrectVMVersionForAnnotationProcessing();
+//			}
+			if (bMain.checkVMVersion(ClassFileConstants.JDK9)) {
+				try {
+					bMain.initRootModules(bMain.batchCompiler.lookupEnvironment, state.getFileSystem());
+				} catch (IllegalArgumentException iae) {
+					ISourceLocation location = null;
+					if (buildConfig.getConfigFile() != null) {
+						location = new SourceLocation(buildConfig.getConfigFile(), 0);
+					}
+					IMessage m = new Message(iae.getMessage(), IMessage.ERROR, null, location);
+					handler.handleMessage(m);
+				}
+			}
+		}
 
 		try {
 			compiler.compile(getCompilationUnits(filenames));
@@ -1057,6 +1128,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	 */
 	public IIntermediateResultsRequestor getInterimResultRequestor() {
 		return new IIntermediateResultsRequestor() {
+			@Override
 			public void acceptResult(InterimCompilationResult result) {
 				if (progressListener != null) {
 					compiledCount++;
@@ -1074,6 +1146,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 
 	public ICompilerRequestor getBatchRequestor() {
 		return new ICompilerRequestor() {
+			@Override
 			public void acceptResult(CompilationResult unitResult) {
 				// end of compile, must now write the results to the output destination
 				// this is either a jar file or a file in a directory
@@ -1318,6 +1391,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 		return "couldn't find aspectjrt.jar on classpath, checked: " + makeClasspathString(buildConfig);
 	}
 
+	@Override
 	public String toString() {
 		StringBuffer buf = new StringBuffer();
 		buf.append("AjBuildManager(");
@@ -1350,6 +1424,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	 *
 	 * @see org.aspectj.ajdt.internal.compiler.AjCompiler.IOutputClassFileNameProvider#getOutputClassFileName(char[])
 	 */
+	@Override
 	public String getOutputClassFileName(char[] eclipseClassFileName, CompilationResult result) {
 		String filename = new String(eclipseClassFileName);
 		filename = filename.replace('/', File.separatorChar) + ".class";
@@ -1373,6 +1448,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	 *
 	 * @see org.eclipse.jdt.internal.compiler.ICompilerAdapterFactory#getAdapter(org.eclipse.jdt.internal.compiler.Compiler)
 	 */
+	@Override
 	public ICompilerAdapter getAdapter(org.aspectj.org.eclipse.jdt.internal.compiler.Compiler forCompiler) {
 		// complete compiler config and return a suitable adapter...
 		populateCompilerOptionsFromLintSettings(forCompiler);
@@ -1427,6 +1503,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 	 *
 	 * @see org.aspectj.ajdt.internal.compiler.IBinarySourceProvider#getBinarySourcesForThisWeave()
 	 */
+	@Override
 	public Map<String, List<UnwovenClassFile>> getBinarySourcesForThisWeave() {
 		return binarySourcesForTheNextCompile;
 	}
@@ -1452,6 +1529,7 @@ public class AjBuildManager implements IOutputClassFileNameProvider, IBinarySour
 
 	private static class AjBuildContexFormatter implements ContextFormatter {
 
+		@Override
 		public String formatEntry(int phaseId, Object data) {
 			StringBuffer sb = new StringBuffer();
 			if (phaseId == CompilationAndWeavingContext.BATCH_BUILD) {
