@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2004, 2013 IBM Corporation and others.
+ * Copyright (c) 2004, 2016 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -91,6 +91,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.docComment.setComment(new String(this.source, start, length));
 	}
 
+	@Override
 	public String toString() {
 		StringBuffer buffer = new StringBuffer();
 		buffer.append("javadoc: ").append(this.docComment).append("\n");	//$NON-NLS-1$ //$NON-NLS-2$
@@ -98,9 +99,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return buffer.toString();
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createArgumentReference(char[], java.lang.Object, int)
-	 */
+	@Override
 	protected Object createArgumentReference(char[] name, int dim, boolean isVarargs, Object typeRef, long[] dimPositions, long argNamePos) throws InvalidInputException {
 		try {
 			MethodRefParameter argument = this.ast.newMethodRefParameter();
@@ -144,7 +143,7 @@ class DocCommentParser extends AbstractCommentParser {
 				}
 			}
 			argument.setType(argType);
-			if (this.ast.apiLevel > AST.JLS8) {
+			if (this.ast.apiLevel > AST.JLS8_INTERNAL) {
 				argument.setVarargs(isVarargs);
 			}
 			argument.setSourceRange(argStart, argEnd - argStart + 1);
@@ -154,9 +153,8 @@ class DocCommentParser extends AbstractCommentParser {
 				throw new InvalidInputException();
 		}
 	}
-/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createFieldReference()
-	 */
+
+	@Override
 	protected Object createFieldReference(Object receiver) throws InvalidInputException {
 		try {
 			MemberRef fieldRef = this.ast.newMemberRef();
@@ -182,9 +180,8 @@ class DocCommentParser extends AbstractCommentParser {
 				throw new InvalidInputException();
 		}
 	}
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createMethodReference(java.lang.Object[])
-	 */
+
+	@Override
 	protected Object createMethodReference(Object receiver, List arguments) throws InvalidInputException {
 		try {
 			// Create method ref
@@ -221,9 +218,7 @@ class DocCommentParser extends AbstractCommentParser {
 		}
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createTag()
-	 */
+	@Override
 	protected void createTag() {
 		TagElement tagElement = this.ast.newTagElement();
 		int position = this.scanner.currentPosition;
@@ -256,9 +251,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.scanner.resetTo(position, this.javadocEnd);
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#createTypeReference()
-	 */
+	@Override
 	protected Object createTypeReference(int primitiveToken) {
 		int size = this.identifierLengthStack[this.identifierLengthPtr];
 		String[] identifiers = new String[size];
@@ -331,9 +324,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return typeRef;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseIdentifierTag(boolean)
-	 */
+	@Override
 	protected boolean parseIdentifierTag(boolean report) {
 		if (super.parseIdentifierTag(report)) {
 			createTag();
@@ -352,9 +343,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#parseTag(int)
-	 */
+	@Override
 	protected boolean parseTag(int previousPosition) throws InvalidInputException {
 
 		// Read tag name
@@ -427,6 +416,9 @@ class DocCommentParser extends AbstractCommentParser {
 						if (length == TAG_CATEGORY_LENGTH && CharOperation.equals(TAG_CATEGORY, tagName)) {
 							this.tagValue = TAG_CATEGORY_VALUE;
 							valid = parseIdentifierTag(false); // TODO (frederic) reconsider parameter value when @category will be significant in spec
+						} else if (length == TAG_CODE_LENGTH && CharOperation.equals(TAG_CODE, tagName)) {
+							this.tagValue = TAG_CODE_VALUE;
+							createTag();
 						} else {
 							this.tagValue = TAG_OTHERS_VALUE;
 							createTag();
@@ -490,8 +482,11 @@ class DocCommentParser extends AbstractCommentParser {
 							this.tagValue = TAG_LINK_VALUE;
 						} else if (length == TAG_LINKPLAIN_LENGTH && CharOperation.equals(TAG_LINKPLAIN, tagName)) {
 							this.tagValue = TAG_LINKPLAIN_VALUE;
+						} else if (length == TAG_LITERAL_LENGTH && CharOperation.equals(TAG_LITERAL, tagName)) {
+							this.tagValue = TAG_LITERAL_VALUE;
 						}
-						if (this.tagValue != NO_TAG_VALUE)  {
+						
+						if (this.tagValue != NO_TAG_VALUE && this.tagValue != TAG_LITERAL_VALUE)  {
 							if (this.inlineTagStarted) {
 								valid = parseReference();
 							} else {
@@ -500,7 +495,7 @@ class DocCommentParser extends AbstractCommentParser {
 								valid = false;
 							}
 						} else {
-							this.tagValue = TAG_OTHERS_VALUE;
+							if (this.tagValue == NO_TAG_VALUE) this.tagValue = TAG_OTHERS_VALUE;
 							createTag();
 						}
 					break;
@@ -589,9 +584,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return valid;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushParamName(java.lang.Object)
-	 */
+	@Override
 	protected boolean pushParamName(boolean isTypeParam) {
 		int idIndex = isTypeParam ? 1 : 0;
 		final SimpleName name = new SimpleName(this.ast);
@@ -627,9 +620,8 @@ class DocCommentParser extends AbstractCommentParser {
 		pushOnAstStack(paramTag, true);
 		return true;
 	}
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushSeeRef(java.lang.Object)
-	 */
+
+	@Override
 	protected boolean pushSeeRef(Object statement) {
 		TagElement seeTag = this.ast.newTagElement();
 		ASTNode node = (ASTNode) statement;
@@ -666,9 +658,8 @@ class DocCommentParser extends AbstractCommentParser {
 		}
 		return true;
 	}
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushText(int, int)
-	 */
+
+	@Override
 	protected void pushText(int start, int end) {
 
 		// Create text element
@@ -713,9 +704,7 @@ class DocCommentParser extends AbstractCommentParser {
 		this.textStart = -1;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#pushThrowName(java.lang.Object)
-	 */
+	@Override
 	protected boolean pushThrowName(Object typeRef) {
 		TagElement throwsTag = this.ast.newTagElement();
 		switch (this.tagValue) {
@@ -732,9 +721,7 @@ class DocCommentParser extends AbstractCommentParser {
 		return true;
 	}
 
-	/* (non-Javadoc)
-	 * @see org.aspectj.org.eclipse.jdt.internal.compiler.parser.AbstractCommentParser#refreshInlineTagPosition(int)
-	 */
+	@Override
 	protected void refreshInlineTagPosition(int previousPosition) {
 		if (this.astPtr != -1) {
 			TagElement previousTag = (TagElement) this.astStack[this.astPtr];
@@ -755,6 +742,7 @@ class DocCommentParser extends AbstractCommentParser {
 	/*
 	 * Add stored tag elements to associated comment.
 	 */
+	@Override
 	protected void updateDocComment() {
 		for (int idx = 0; idx <= this.astPtr; idx++) {
 			this.docComment.tags().add(this.astStack[idx]);
