@@ -1,5 +1,6 @@
+// ASPECTJ
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -81,6 +82,7 @@ public abstract class AbstractMethodDeclaration
 	/*
 	 *	We cause the compilation task to abort to a given extent.
 	 */
+	@Override
 	public void abort(int abortLevel, CategorizedProblem problem) {
 
 		switch (abortLevel) {
@@ -203,17 +205,31 @@ public abstract class AbstractMethodDeclaration
 	/**
 	 * Feed null information from argument annotations into the analysis and mark arguments as assigned.
 	 */
-	static void analyseArguments(FlowInfo flowInfo, Argument[] methodArguments, MethodBinding methodBinding) {
+	static void analyseArguments(LookupEnvironment environment, FlowInfo flowInfo, Argument[] methodArguments, MethodBinding methodBinding) {
 		if (methodArguments != null) {
-			for (int i = 0, count = methodArguments.length; i < count; i++) {
-				if (methodBinding.parameterNonNullness != null) {
-					// leverage null-info from parameter annotations:
-					Boolean nonNullNess = methodBinding.parameterNonNullness[i];
-					if (nonNullNess != null) {
-						if (nonNullNess.booleanValue())
-							flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
-						else
-							flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
+			boolean usesNullTypeAnnotations = environment.usesNullTypeAnnotations();
+			int length = Math.min(methodBinding.parameters.length, methodArguments.length);
+			for (int i = 0; i < length; i++) {
+				if (usesNullTypeAnnotations) {
+					// leverage null type annotations:
+					long tagBits = methodBinding.parameters[i].tagBits & TagBits.AnnotationNullMASK;
+					if (tagBits == TagBits.AnnotationNonNull)
+						flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
+					else if (tagBits == TagBits.AnnotationNullable)
+						flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
+					else if (methodBinding.parameters[i].isFreeTypeVariable()) {
+						flowInfo.markNullStatus(methodArguments[i].binding, FlowInfo.FREE_TYPEVARIABLE);
+					}
+				} else {					
+					if (methodBinding.parameterNonNullness != null) {
+						// leverage null-info from parameter annotations:
+						Boolean nonNullNess = methodBinding.parameterNonNullness[i];
+						if (nonNullNess != null) {
+							if (nonNullNess.booleanValue())
+								flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
+							else
+								flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
+						}
 					}
 				}
 				// tag parameters as being set:
@@ -222,27 +238,7 @@ public abstract class AbstractMethodDeclaration
 		}
 	}
 
-	/**
-	 * Feed null information from argument annotations into the analysis and mark arguments as assigned.
-	 * Variant for Java 8 using type annotations
-	 */
-	static void analyseArguments18(FlowInfo flowInfo, Argument[] methodArguments, MethodBinding methodBinding) {
-		if (methodArguments != null) {
-			int length = Math.min(methodBinding.parameters.length, methodArguments.length);
-			for (int i = 0; i < length; i++) {
-				// leverage null type annotations:
-				long tagBits = methodBinding.parameters[i].tagBits & TagBits.AnnotationNullMASK;
-				if (tagBits == TagBits.AnnotationNonNull)
-					flowInfo.markAsDefinitelyNonNull(methodArguments[i].binding);
-				else if (tagBits == TagBits.AnnotationNullable)
-					flowInfo.markPotentiallyNullBit(methodArguments[i].binding);
-	
-				// tag parameters as being set:
-				flowInfo.markAsDefinitelyAssigned(methodArguments[i].binding);
-			}
-		}
-	}
-
+	@Override
 	public CompilationResult compilationResult() {
 
 		return this.compilationResult;
@@ -394,6 +390,7 @@ public abstract class AbstractMethodDeclaration
 		}
 	}
 
+	@Override
 	public CompilationUnitDeclaration getCompilationUnitDeclaration() {
 		if (this.scope != null) {
 			return this.scope.compilationUnitScope().referenceContext;
@@ -401,6 +398,7 @@ public abstract class AbstractMethodDeclaration
 		return null;
 	}
 
+	@Override
 	public boolean hasErrors() {
 		return this.ignoreFurtherInvestigation;
 	}
@@ -467,6 +465,7 @@ public abstract class AbstractMethodDeclaration
 	 */
 	public abstract void parseStatements(Parser parser, CompilationUnitDeclaration unit);
 
+	@Override
 	public StringBuffer print(int tab, StringBuffer output) {
 
 		if (this.javadoc != null) {
@@ -546,7 +545,7 @@ public abstract class AbstractMethodDeclaration
 			resolveReceiver();
 			bindThrownExceptions();
 			resolveJavadoc();
-			resolveAnnotations(this.scope, this.annotations, this.binding);
+			resolveAnnotations(this.scope, this.annotations, this.binding, this.isConstructor());
 			
 			long sourceLevel = this.scope.compilerOptions().sourceLevel;
 			if (sourceLevel < ClassFileConstants.JDK1_8) // otherwise already checked via Argument.createBinding
@@ -651,10 +650,12 @@ public abstract class AbstractMethodDeclaration
 		}
 	}
 
+	@Override
 	public void tagAsHavingErrors() {
 		this.ignoreFurtherInvestigation = true;
 	}
 	
+	@Override
 	public void tagAsHavingIgnoredMandatoryErrors(int problemId) {
 		// Nothing to do for this context;
 	}
