@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -26,6 +26,8 @@
  *								Bug 428274 - [1.8] [compiler] Cannot cast from Number to double
  *								Bug 428352 - [1.8][compiler] Resolution errors don't always surface
  *								Bug 452788 - [1.8][compiler] Type not correctly inferred in lambda expression
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.ast;
 
@@ -74,7 +76,7 @@ public abstract class Expression extends Statement {
 	public int implicitConversion;
 	public TypeBinding resolvedType;
 	
-	static Expression [] NO_EXPRESSIONS = new Expression[0];
+	public static Expression [] NO_EXPRESSIONS = new Expression[0];
 	
 
 public static final boolean isConstantValueRepresentable(Constant constant, int constantTypeID, int targetTypeID) {
@@ -232,6 +234,7 @@ public Expression() {
 	super();
 }
 
+@Override
 public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo) {
 	return flowInfo;
 }
@@ -571,9 +574,10 @@ public final boolean checkCastTypesCompatibility(Scope scope, TypeBinding castTy
  * @param scope the scope of the analysis
  * @param flowContext the current flow context
  * @param flowInfo the upstream flow info; caveat: may get modified
+ * @param ttlForFieldCheck if this is a reference to a field we will mark that field as nonnull for the specified timeToLive
  * @return could this expression be checked by the current implementation?
  */
-public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
+public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo, int ttlForFieldCheck) {
 	boolean isNullable = false;
 	if (this.resolvedType != null) {
 		// 1. priority: @NonNull
@@ -606,6 +610,9 @@ public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flow
 	}
 	return false; // not checked
 }
+public boolean checkNPE(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
+	return checkNPE(scope, flowContext, flowInfo, 0); // default: don't mark field references as checked for null
+}
 
 /** If this expression requires unboxing check if that operation can throw NPE. */
 protected void checkNPEbyUnboxing(BlockScope scope, FlowContext flowContext, FlowInfo flowInfo) {
@@ -657,7 +664,7 @@ public void computeConversion(Scope scope, TypeBinding runtimeType, TypeBinding
 		TypeBinding boxedType = scope.environment().computeBoxingType(runtimeType);
 		if (TypeBinding.equalsEquals(boxedType, runtimeType)) // Object o = 12;
 			boxedType = compileTimeType;
-		if (boxedType.id >= TypeIds.T_LastWellKnownTypeId) {  // (Comparable & Serializable) 0
+		if (boxedType.id > TypeIds.T_JavaLangBoolean) {       // (Comparable & Serializable) 0
 			boxedType = compileTimeType;
 		}
 		this.implicitConversion = TypeIds.BOXING | (boxedType.id << 4) + compileTimeType.id;
@@ -706,6 +713,7 @@ public void computeConversion(Scope scope, TypeBinding runtimeType, TypeBinding
  * @param currentScope org.aspectj.org.eclipse.jdt.internal.compiler.lookup.BlockScope
  * @param codeStream org.aspectj.org.eclipse.jdt.internal.compiler.codegen.CodeStream
  */
+@Override
 public void generateCode(BlockScope currentScope, CodeStream codeStream) {
 	if ((this.bits & ASTNode.IsReachable) == 0) {
 		return;
@@ -843,7 +851,7 @@ public void generateOptimizedStringConcatenationCreation(BlockScope blockScope,
 }
 
 private MethodBinding[] getAllOriginalInheritedMethods(ReferenceBinding binding) {
-	ArrayList<MethodBinding> collector = new ArrayList<MethodBinding>();
+	ArrayList<MethodBinding> collector = new ArrayList<>();
 	getAllInheritedMethods0(binding, collector);
 	for (int i = 0, len = collector.size(); i < len; i++) {
 		collector.set(i, collector.get(i).original());
@@ -1004,6 +1012,7 @@ public TypeBinding postConversionType(Scope scope) {
 	return convertedType;
 }
 
+@Override
 public StringBuffer print(int indent, StringBuffer output) {
 	printIndent(indent, output);
 	return printExpression(indent, output);
@@ -1011,10 +1020,12 @@ public StringBuffer print(int indent, StringBuffer output) {
 
 public abstract StringBuffer printExpression(int indent, StringBuffer output);
 
+@Override
 public StringBuffer printStatement(int indent, StringBuffer output) {
 	return print(indent, output).append(";"); //$NON-NLS-1$
 }
 
+@Override
 public void resolve(BlockScope scope) {
 	// drops the returning expression's type whatever the type is.
 	this.resolveType(scope);
@@ -1081,7 +1092,10 @@ public boolean forcedToBeRaw(ReferenceContext referenceContext) {
 			if (field.type.isRawType()) {
 				if (referenceContext instanceof AbstractMethodDeclaration) {
 					AbstractMethodDeclaration methodDecl = (AbstractMethodDeclaration) referenceContext;
-					if (TypeBinding.notEquals(field.declaringClass, methodDecl.binding.declaringClass)) { // inherited raw field, see https://bugs.eclipse.org/bugs/show_bug.cgi?id=337962
+					ReferenceBinding declaringClass = methodDecl.binding != null 
+							? methodDecl.binding.declaringClass
+							: methodDecl.scope.enclosingReceiverType();
+					if (TypeBinding.notEquals(field.declaringClass, declaringClass)) { // inherited raw field, see https://bugs.eclipse.org/bugs/show_bug.cgi?id=337962
 						return true;
 					}
 				} else if (referenceContext instanceof TypeDeclaration) {
@@ -1213,6 +1227,7 @@ public Expression toTypeReference() {
  * @param visitor
  * @param scope
  */
+@Override
 public void traverse(ASTVisitor visitor, BlockScope scope) {
 	// nothing to do
 }
