@@ -32,9 +32,7 @@ public long elapsedTime() {
 	return this.endTime - this.startTime;
 }
 
-/**
- * @see java.lang.Comparable#compareTo(java.lang.Object)
- */
+@Override
 public int compareTo(Object o) {
 	CompilerStats otherStats = (CompilerStats) o;
 	long time1 = elapsedTime();
