@@ -50,6 +50,8 @@ public class CaptureBinding18 extends CaptureBinding {
 					this.superclass = (ReferenceBinding) aBound;
 				else if (aBound.isInterface())
 					numReferenceInterfaces++;
+			} else if (TypeBinding.equalsEquals(aBound.leafComponentType(), this)) {
+				return false; // cycle detected
 			}
 		}
 		this.superInterfaces = new ReferenceBinding[numReferenceInterfaces];
@@ -64,20 +66,24 @@ public class CaptureBinding18 extends CaptureBinding {
 		return true;
 	}
 
+	@Override
 	public void initializeBounds(Scope scope, ParameterizedTypeBinding capturedParameterizedType) {
 		// nothing to initialize here (and cannot use super methods which requires wildcard to be set).
 	}
 
+	@Override
 	public TypeBinding clone(TypeBinding enclosingType) {
 		return new CaptureBinding18(this);
 	}
 
+	@Override
 	public MethodBinding[] getMethods(char[] selector) {
 		if (this.upperBounds.length == 1 && this.upperBounds[0] instanceof ReferenceBinding)
 			return ((ReferenceBinding)this.upperBounds[0]).getMethods(selector);
 		return super.getMethods(selector);
 	}
 
+	@Override
 	public TypeBinding erasure() {
 		if (this.upperBounds != null && this.upperBounds.length > 1) {
 			ReferenceBinding[] erasures = new ReferenceBinding[this.upperBounds.length];
@@ -99,6 +105,7 @@ public class CaptureBinding18 extends CaptureBinding {
 	/**
 	 * @see TypeBinding#isEquivalentTo(TypeBinding)
 	 */
+	@Override
 	public boolean isEquivalentTo(TypeBinding otherType) {
 		// from CaptureBinding:
 		if (equalsEquals(this, otherType)) return true;
@@ -123,6 +130,7 @@ public class CaptureBinding18 extends CaptureBinding {
 		return false;
 	}
 
+	@Override
 	public boolean isCompatibleWith(TypeBinding otherType, Scope captureScope) {
 		if (TypeBinding.equalsEquals(this, otherType))
 			return true;
@@ -173,6 +181,7 @@ public class CaptureBinding18 extends CaptureBinding {
 		}
 	}
 
+	@Override
 	public TypeBinding findSuperTypeOriginatingFrom(TypeBinding otherType) {
 		if (this.upperBounds != null && this.upperBounds.length > 1) {
 			for (int i = 0; i < this.upperBounds.length; i++) {
@@ -186,6 +195,7 @@ public class CaptureBinding18 extends CaptureBinding {
 		return super.findSuperTypeOriginatingFrom(otherType);
 	}
 
+	@Override
 	TypeBinding substituteInferenceVariable(InferenceVariable var, TypeBinding substituteType) {
 		if (this.inRecursiveFunction) return this;
 		this.inRecursiveFunction = true;
@@ -267,6 +277,7 @@ public class CaptureBinding18 extends CaptureBinding {
 		}
 	}
 
+	@Override
 	public boolean isProperType(boolean admitCapture18) {
 		if (!admitCapture18) 
 			return false;
@@ -290,28 +301,7 @@ public class CaptureBinding18 extends CaptureBinding {
 
 	int recursionLevel = 0; // used to give a hint at recursive types without going into infinity
 
-	public char[] genericTypeSignature() {
-		// since we have no wildcard, we combine the logic from CaptureBinding plus WildcardBinding here:
-		if (this.genericTypeSignature == null) {
-			char[] boundSignature;
-			try {
-				if (this.prototype.recursionLevel++ > 0 || this.firstBound == null) {
-					boundSignature = TypeConstants.WILDCARD_STAR;
-				} else if (this.upperBounds != null) {
-					boundSignature = CharOperation.concat(TypeConstants.WILDCARD_PLUS, this.firstBound.genericTypeSignature());
-				} else if (this.lowerBound != null) {
-					boundSignature = CharOperation.concat(TypeConstants.WILDCARD_MINUS, this.lowerBound.genericTypeSignature());
-				} else {
-					boundSignature = TypeConstants.WILDCARD_STAR;
-				}
-				this.genericTypeSignature = CharOperation.concat(TypeConstants.WILDCARD_CAPTURE, boundSignature);
-			} finally {
-				this.prototype.recursionLevel--;
-			}
-		}
-		return this.genericTypeSignature;
-	}
-	
+	@Override
 	public char[] readableName() {
 		if (this.lowerBound == null && this.firstBound != null) {
 			if (this.prototype.recursionLevel < 2) {
@@ -338,6 +328,7 @@ public class CaptureBinding18 extends CaptureBinding {
 		return super.readableName();
 	}
 
+	@Override
 	public char[] shortReadableName() {
 		if (this.lowerBound == null && this.firstBound != null) {
 			if (this.prototype.recursionLevel < 2) {
