@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2013, 2015 GK Software AG.
+ * Copyright (c) 2013, 2016 GK Software AG.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -7,6 +7,8 @@
  *
  * Contributors:
  *     Stephan Herrmann - initial API and implementation
+ *     Lars Vogel <Lars.Vogel@vogella.com> - Contributions for
+ *     						Bug 473178
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.compiler.lookup;
 
@@ -56,6 +58,7 @@ class ConstraintTypeFormula extends ConstraintFormula {
 	ConstraintTypeFormula() { }
 
 	// return: ReductionResult or ConstraintFormula[]
+	@Override
 	public Object reduce(InferenceContext18 inferenceContext) {
 		switch (this.relation) {
 		case COMPATIBLE:
@@ -99,13 +102,16 @@ class ConstraintTypeFormula extends ConstraintFormula {
 				if (!checkIVFreeTVmatch(this.left, this.right))
 					checkIVFreeTVmatch(this.right, this.left);
 			// 18.2.4:
-			return reduceTypeEquality(inferenceContext.object);
+			return reduceTypeEquality(inferenceContext.object, inferenceContext);
 		case TYPE_ARGUMENT_CONTAINED:
 			// 18.2.3:
 			if (this.right.kind() != Binding.WILDCARD_TYPE) { // "If T is a type" ... all alternatives require "wildcard"
 				if (this.left.kind() != Binding.WILDCARD_TYPE) {
 					return ConstraintTypeFormula.create(this.left, this.right, SAME, this.isSoft);						
 				} else {
+					// TODO: speculative addition:
+					if (this.right instanceof InferenceVariable)
+						return new TypeBound((InferenceVariable) this.right, this.left, SAME, this.isSoft);
 					return FALSE;
 				}
 			} else {
@@ -155,7 +161,7 @@ class ConstraintTypeFormula extends ConstraintFormula {
 		return false;
 	}
 
-	private Object reduceTypeEquality(TypeBinding object) {
+	private Object reduceTypeEquality(TypeBinding object, InferenceContext18 inferenceContext) {
 		// 18.2.4
 		if (this.left.kind() == Binding.WILDCARD_TYPE) {
 			if (this.right.kind() == Binding.WILDCARD_TYPE) {
@@ -182,10 +188,13 @@ class ConstraintTypeFormula extends ConstraintFormula {
 						return TRUE;
 					return FALSE;
 				}
-				if (this.left instanceof InferenceVariable) {
+				if (this.left.id == TypeIds.T_null || this.right.id== TypeIds.T_null) {
+					return FALSE;
+				}
+				if (this.left instanceof InferenceVariable && !this.right.isPrimitiveType()) {
 					return new TypeBound((InferenceVariable) this.left, this.right, SAME, this.isSoft);
 				}
-				if (this.right instanceof InferenceVariable) {
+				if (this.right instanceof InferenceVariable && !this.left.isPrimitiveType()) {
 					return new TypeBound((InferenceVariable) this.right, this.left, SAME, this.isSoft);
 				}
 				if ((this.left.isClass() || this.left.isInterface()) 
@@ -205,19 +214,31 @@ class ConstraintTypeFormula extends ConstraintFormula {
 					}
 					return constraints;
 				}
-				if (this.left.isArrayType() && this.right.isArrayType() && this.left.dimensions() == this.right.dimensions()) {
-					// checking dimensions already now is an optimization over reducing one dim at a time
-					return ConstraintTypeFormula.create(this.left.leafComponentType(), this.right.leafComponentType(), SAME, this.isSoft);
+				if (this.left.isArrayType() && this.right.isArrayType()) {
+					if (this.left.dimensions() == this.right.dimensions()) {
+						// checking dimensions in one step is an optimization over reducing one dim at a time
+						return ConstraintTypeFormula.create(this.left.leafComponentType(), this.right.leafComponentType(), SAME, this.isSoft);
+					} else if (this.left.dimensions() > 0 && this.right.dimensions() > 0) {
+						TypeBinding leftPrime = peelOneDimension(this.left, inferenceContext.environment);
+						TypeBinding rightPrime = peelOneDimension(this.right, inferenceContext.environment);
+						return ConstraintTypeFormula.create(leftPrime, rightPrime, SAME, this.isSoft);
+					}
 				}
 			}
 		}
 		return FALSE;
 	}
 
+	private TypeBinding peelOneDimension(TypeBinding arrayType, LookupEnvironment env) {
+		if (arrayType.dimensions() == 1)
+			return arrayType.leafComponentType();
+		return env.createArrayType(arrayType.leafComponentType(), arrayType.dimensions()-1);
+	}
+
 	private Object reduceSubType(Scope scope, TypeBinding subCandidate, TypeBinding superCandidate) {
 		// 18.2.3 Subtyping Constraints
 		if (subCandidate.isProperType(true) && superCandidate.isProperType(true)) {
-			if (subCandidate.isCompatibleWith(superCandidate, scope))
+			if (subCandidate.isSubtypeOf(superCandidate, InferenceContext18.SIMULATE_BUG_JDK_8026527))
 				return TRUE;
 			return FALSE;
 		}
@@ -234,13 +255,13 @@ class ConstraintTypeFormula extends ConstraintFormula {
 			case Binding.TYPE:
 			case Binding.RAW_TYPE:
 				{
-					if (subCandidate.isSubtypeOf(superCandidate))
+					if (subCandidate.isSubtypeOf(superCandidate, InferenceContext18.SIMULATE_BUG_JDK_8026527))
 						return TRUE;
 					return FALSE;
 				}
 			case Binding.PARAMETERIZED_TYPE:
 				{
-					List<ConstraintFormula> constraints = new ArrayList<ConstraintFormula>();
+					List<ConstraintFormula> constraints = new ArrayList<>();
 					while (superCandidate != null && superCandidate.kind() == Binding.PARAMETERIZED_TYPE && subCandidate != null)  {
 						if (!addConstraintsFromTypeParameters(subCandidate, (ParameterizedTypeBinding) superCandidate, constraints))
 							return FALSE;
@@ -382,6 +403,7 @@ class ConstraintTypeFormula extends ConstraintFormula {
 					TypeBinding.equalsEquals(this.left, that.left) && TypeBinding.equalsEquals(this.right, that.right));
 	}
 	
+	@Override
 	public boolean applySubstitution(BoundSet solutionSet, InferenceVariable[] variables) {
 		super.applySubstitution(solutionSet, variables);
 		for (int i=0; i<variables.length; i++) {
@@ -395,6 +417,7 @@ class ConstraintTypeFormula extends ConstraintFormula {
 	}
 
 	// debugging
+	@Override
 	public String toString() {
 		StringBuffer buf = new StringBuffer("Type Constraint:\n"); //$NON-NLS-1$
 		buf.append('\t').append(LEFT_ANGLE_BRACKET);
