@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2011 IBM Corporation and others.
+ * Copyright (c) 2005, 2015 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -42,6 +42,7 @@ public class ScannerHelper {
 	private static long[][][] Tables;
 	private static long[][][] Tables7;
 	private static long[][][] Tables8;
+	private static long[][][] Tables9;
 
 	public final static int MAX_OBVIOUS = 128;
 	public final static int[] OBVIOUS_IDENT_CHAR_NATURES = new int[MAX_OBVIOUS];
@@ -140,6 +141,9 @@ static void initializeTable17() {
 static void initializeTable18() {
 	Tables8 = initializeTables("unicode6_2"); //$NON-NLS-1$
 }
+static void initializeTable19() {
+	Tables9 = initializeTables("unicode8"); //$NON-NLS-1$
+}
 static long[][][] initializeTables(String unicode_path) {
 	long[][][] tempTable = new long[2][][];
 	tempTable[START_INDEX] = new long[3][];
@@ -286,8 +290,8 @@ public static boolean isJavaIdentifierPart(long complianceLevel, int codePoint)
 			case 14 :
 				return isBitSet(Tables7[PART_INDEX][3], codePoint & 0xFFFF);
 		}
-	} else {
-		// java 7 supports Unicode 6.2
+	} else if (complianceLevel <= ClassFileConstants.JDK1_8) {
+		// java 8 supports Unicode 6.2
 		if (Tables8 == null) {
 			initializeTable18();
 		}
@@ -301,6 +305,21 @@ public static boolean isJavaIdentifierPart(long complianceLevel, int codePoint)
 			case 14 :
 				return isBitSet(Tables8[PART_INDEX][3], codePoint & 0xFFFF);
 		}
+	} else {
+		// java 9 supports Unicode 8
+		if (Tables9 == null) {
+			initializeTable19();
+		}
+		switch((codePoint & 0x1F0000) >> 16) {
+			case 0 :
+				return isBitSet(Tables9[PART_INDEX][0], codePoint & 0xFFFF);
+			case 1 :
+				return isBitSet(Tables9[PART_INDEX][1], codePoint & 0xFFFF);
+			case 2 :
+				return isBitSet(Tables9[PART_INDEX][2], codePoint & 0xFFFF);
+			case 14 :
+				return isBitSet(Tables9[PART_INDEX][3], codePoint & 0xFFFF);
+		}
 	}
 	return false;
 }
@@ -348,7 +367,7 @@ public static boolean isJavaIdentifierStart(long complianceLevel, int codePoint)
 			case 2 :
 				return isBitSet(Tables7[START_INDEX][2], codePoint & 0xFFFF);
 		}
-	} else {
+	} else if (complianceLevel <= ClassFileConstants.JDK1_8) {
 		// java 7 supports Unicode 6
 		if (Tables8 == null) {
 			initializeTable18();
@@ -361,6 +380,19 @@ public static boolean isJavaIdentifierStart(long complianceLevel, int codePoint)
 			case 2 :
 				return isBitSet(Tables8[START_INDEX][2], codePoint & 0xFFFF);
 		}
+	} else {
+		// java 9 supports Unicode 7
+		if (Tables9 == null) {
+			initializeTable19();
+		}
+		switch((codePoint & 0x1F0000) >> 16) {
+			case 0 :
+				return isBitSet(Tables9[START_INDEX][0], codePoint & 0xFFFF);
+			case 1 :
+				return isBitSet(Tables9[START_INDEX][1], codePoint & 0xFFFF);
+			case 2 :
+				return isBitSet(Tables9[START_INDEX][2], codePoint & 0xFFFF);
+		}
 	}
 	return false;
 }
