@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2017 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -8,6 +8,7 @@
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *     Vladimir Piskarev <pisv@1c.ru> - Building large Java element deltas is really slow - https://bugs.eclipse.org/443928
+ *     Vladimir Piskarev <pisv@1c.ru> - F_CONTENT sometimes lost when merging deltas - https://bugs.eclipse.org/520336
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.core;
 
@@ -81,6 +82,8 @@ public class JavaElementDelta extends SimpleDelta implements IJavaElementDelta {
 	 */
 	Map<Key, Integer> childIndex;
 
+	public boolean ignoreFromTests = false;
+
 	/**
 	 * The delta key
 	 */
@@ -90,9 +93,11 @@ public class JavaElementDelta extends SimpleDelta implements IJavaElementDelta {
 		public Key(IJavaElement element) {
 			this.element = element;
 		}
+		@Override
 		public int hashCode() {
 			return this.element.hashCode();
 		}
+		@Override
 		public boolean equals(Object obj) {
 			if (!(obj instanceof Key))
 				return false;
@@ -181,16 +186,13 @@ protected void addAffectedChild(JavaElementDelta child) {
 						}
 
 						// update flags
-						boolean childHadContentFlag = (child.changeFlags & F_CONTENT) != 0;
-						boolean existingChildHadChildrenFlag = (existingChild.changeFlags & F_CHILDREN) != 0;
-						existingChild.changeFlags |= child.changeFlags;
-
-						// remove F_CONTENT flag if existing child had F_CHILDREN flag set
-						// (case of fine grained delta (existing child) and delta coming from
-						// DeltaProcessor (child))
-						if (childHadContentFlag && existingChildHadChildrenFlag) {
-							existingChild.changeFlags &= ~F_CONTENT;
+						int flags = child.changeFlags;
+						// case of fine grained delta (existing child) and delta coming from
+						// DeltaProcessor (child): ensure F_CONTENT is not propagated from child
+						if ((existingChild.changeFlags & F_FINE_GRAINED) != 0 && (flags & F_FINE_GRAINED) == 0) {
+							flags &= ~F_CONTENT;
 						}
+						existingChild.changeFlags |= flags;
 
 						// add the non-java resource deltas if needed
 						// note that the child delta always takes precedence over this existing child delta
@@ -372,12 +374,14 @@ public void fineGrained() {
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElementDelta[] getAddedChildren() {
 	return getChildrenOfType(ADDED);
 }
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElementDelta[] getAffectedChildren() {
 	return this.affectedChildren;
 }
@@ -403,15 +407,18 @@ private ArrayList getAncestors(IJavaElement element) {
 	parents.trimToSize();
 	return parents;
 }
+@Override
 public CompilationUnit getCompilationUnitAST() {
 	return this.ast;
 }
+@Override
 public IJavaElementDelta[] getAnnotationDeltas() {
 	return this.annotationDeltas;
 }
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElementDelta[] getChangedChildren() {
 	return getChildrenOfType(CHANGED);
 }
@@ -467,30 +474,35 @@ protected JavaElementDelta getDeltaFor(IJavaElement element) {
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElement getElement() {
 	return this.changedElement;
 }
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElement getMovedFromElement() {
 	return this.movedFromHandle;
 }
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElement getMovedToElement() {
 	return this.movedToHandle;
 }
 /**
  * @see IJavaElementDelta
  */
+@Override
 public IJavaElementDelta[] getRemovedChildren() {
 	return getChildrenOfType(REMOVED);
 }
 /**
  * Return the collection of resource deltas. Return null if none.
  */
+@Override
 public IResourceDelta[] getResourceDeltas() {
 	if (this.resourceDeltas == null) return null;
 	if (this.resourceDeltas.length != this.resourceDeltasCounter) {
@@ -689,6 +701,7 @@ public String toDebugString(int depth) {
 	}
 	return buffer.toString();
 }
+@Override
 protected boolean toDebugString(StringBuffer buffer, int flags) {
 	boolean prev = super.toDebugString(buffer, flags);
 
@@ -818,6 +831,7 @@ protected boolean toDebugString(StringBuffer buffer, int flags) {
  * Returns a string representation of this delta's
  * structure suitable for debug purposes.
  */
+@Override
 public String toString() {
 	return toDebugString(0);
 }
