@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2013 IBM Corporation and others.
+ * Copyright (c) 2000, 2018 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -24,6 +24,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleSet;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SuffixConstants;
+import org.aspectj.org.eclipse.jdt.internal.core.CompilationGroup;
 import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager;
 import org.aspectj.org.eclipse.jdt.internal.core.PackageFragment;
 import org.aspectj.org.eclipse.jdt.internal.core.util.Messages;
@@ -50,13 +51,13 @@ protected BuildNotifier notifier;
 
 protected Compiler compiler;
 protected WorkQueue workQueue;
-protected ArrayList problemSourceFiles;
+protected LinkedHashSet<SourceFile> problemSourceFiles;
 protected boolean compiledAllAtOnce;
 
 private boolean inCompiler;
 
 protected boolean keepStoringProblemMarkers;
-protected SimpleSet filesWithAnnotations = null;
+protected Set<SourceFile> filesWithAnnotations = null;
 
 //2000 is best compromise between space used and speed
 public static int MAX_AT_ONCE = Integer.getInteger(JavaModelManager.MAX_COMPILED_UNITS_AT_ONCE, 2000).intValue();
@@ -80,16 +81,19 @@ public final static String[] JAVA_TASK_MARKER_ATTRIBUTE_NAMES = {
 	IMarker.USER_EDITABLE,
 	IMarker.SOURCE_ID,
 };
-public final static Integer S_ERROR = new Integer(IMarker.SEVERITY_ERROR);
-public final static Integer S_WARNING = new Integer(IMarker.SEVERITY_WARNING);
-public final static Integer P_HIGH = new Integer(IMarker.PRIORITY_HIGH);
-public final static Integer P_NORMAL = new Integer(IMarker.PRIORITY_NORMAL);
-public final static Integer P_LOW = new Integer(IMarker.PRIORITY_LOW);
+public final static Integer S_ERROR = Integer.valueOf(IMarker.SEVERITY_ERROR);
+public final static Integer S_WARNING = Integer.valueOf(IMarker.SEVERITY_WARNING);
+public final static Integer S_INFO = Integer.valueOf(IMarker.SEVERITY_INFO);
+public final static Integer P_HIGH = Integer.valueOf(IMarker.PRIORITY_HIGH);
+public final static Integer P_NORMAL = Integer.valueOf(IMarker.PRIORITY_NORMAL);
+public final static Integer P_LOW = Integer.valueOf(IMarker.PRIORITY_LOW);
+private CompilationGroup compilationGroup;
 
-protected AbstractImageBuilder(JavaBuilder javaBuilder, boolean buildStarting, State newState) {
+protected AbstractImageBuilder(JavaBuilder javaBuilder, boolean buildStarting, State newState, CompilationGroup compilationGroup) {
 	// local copies
 	this.javaBuilder = javaBuilder;
-	this.nameEnvironment = javaBuilder.nameEnvironment;
+	this.compilationGroup = compilationGroup;
+	this.nameEnvironment = compilationGroup == CompilationGroup.TEST ? javaBuilder.testNameEnvironment : javaBuilder.nameEnvironment;
 	this.sourceLocations = this.nameEnvironment.sourceLocations;
 	this.notifier = javaBuilder.notifier;
 	this.keepStoringProblemMarkers = true; // may get disabled when missing classfiles are encountered
@@ -98,7 +102,7 @@ protected AbstractImageBuilder(JavaBuilder javaBuilder, boolean buildStarting, S
 		this.newState = newState == null ? new State(javaBuilder) : newState;
 		this.compiler = newCompiler();
 		this.workQueue = new WorkQueue();
-		this.problemSourceFiles = new ArrayList(3);
+		this.problemSourceFiles = new LinkedHashSet(3);
 
 		if (this.javaBuilder.participants != null) {
 			for (int i = 0, l = this.javaBuilder.participants.length; i < l; i++) {
@@ -106,7 +110,7 @@ protected AbstractImageBuilder(JavaBuilder javaBuilder, boolean buildStarting, S
 					// initialize this set so the builder knows to gather CUs that define Annotation types
 					// each Annotation processor participant is then asked to process these files AFTER
 					// the compile loop. The normal dependency loop will then recompile all affected types
-					this.filesWithAnnotations = new SimpleSet(1);
+					this.filesWithAnnotations = new HashSet<>(1);
 					break;
 				}
 			}
@@ -114,6 +118,7 @@ protected AbstractImageBuilder(JavaBuilder javaBuilder, boolean buildStarting, S
 	}
 }
 
+@Override
 public void acceptResult(CompilationResult result) {
 	// In Batch mode, we write out the class files, hold onto the dependency info
 	// & additional types and report problems.
@@ -123,7 +128,13 @@ public void acceptResult(CompilationResult result) {
 	// Before reporting the new problems, we need to update the problem count &
 	// remove the old problems. Plus delete additional class files that no longer exist.
 
-	SourceFile compilationUnit = (SourceFile) result.getCompilationUnit(); // go directly back to the sourceFile
+	ICompilationUnit resultCU = result.getCompilationUnit();
+	if (!(resultCU instanceof SourceFile)) {
+		return; // can happen for secondary module redirected via CompilationUnit
+		// we should never have to report errors etc for those, but this entire construction is a kludge,
+		// working around lack of support for modules in SourceTypeConverter
+	}
+	SourceFile compilationUnit = (SourceFile) resultCU; // go directly back to the sourceFile
 	if (!this.workQueue.isCompiled(compilationUnit)) {
 		this.workQueue.finished(compilationUnit);
 
@@ -136,8 +147,7 @@ public void acceptResult(CompilationResult result) {
 
 		if (result.hasInconsistentToplevelHierarchies)
 			// ensure that this file is always retrieved from source for the rest of the build
-			if (!this.problemSourceFiles.contains(compilationUnit))
-				this.problemSourceFiles.add(compilationUnit);
+			this.problemSourceFiles.add(compilationUnit);
 
 		IType mainType = null;
 		String mainTypeName = null;
@@ -208,8 +218,7 @@ public void acceptResult(CompilationResult result) {
 protected void acceptSecondaryType(ClassFile classFile) {
 	// noop
 }
-
-protected void addAllSourceFiles(final ArrayList sourceFiles) throws CoreException {
+protected void addAllSourceFiles(final LinkedHashSet<SourceFile> sourceFiles) throws CoreException {
 	for (int i = 0, l = this.sourceLocations.length; i < l; i++) {
 		final ClasspathMultiDirectory sourceLocation = this.sourceLocations[i];
 		final char[][] exclusionPatterns = sourceLocation.exclusionPatterns;
@@ -220,6 +229,7 @@ protected void addAllSourceFiles(final ArrayList sourceFiles) throws CoreExcepti
 		final boolean isOutputFolder = sourceLocation.sourceFolder.equals(outputFolder);
 		sourceLocation.sourceFolder.accept(
 			new IResourceProxyVisitor() {
+				@Override
 				public boolean visit(IResourceProxy proxy) throws CoreException {
 					switch(proxy.getType()) {
 						case IResource.FILE :
@@ -228,7 +238,8 @@ protected void addAllSourceFiles(final ArrayList sourceFiles) throws CoreExcepti
 								if (exclusionPatterns != null || inclusionPatterns != null)
 									if (Util.isExcluded(resource.getFullPath(), inclusionPatterns, exclusionPatterns, false))
 										return false;
-								sourceFiles.add(new SourceFile((IFile) resource, sourceLocation));
+								SourceFile unit = new SourceFile((IFile) resource, sourceLocation);
+								sourceFiles.add(unit);
 							}
 							return false;
 						case IResource.FOLDER :
@@ -282,7 +293,7 @@ protected void cleanUp() {
 * if they are affected by the changes.
 */
 protected void compile(SourceFile[] units) {
-	if (this.filesWithAnnotations != null && this.filesWithAnnotations.elementSize > 0)
+	if (this.filesWithAnnotations != null && this.filesWithAnnotations.size() > 0)
 		// will add files that have annotations in acceptResult() & then processAnnotations() before exitting this method
 		this.filesWithAnnotations.clear();
 
@@ -354,12 +365,17 @@ protected void compile(SourceFile[] units, SourceFile[] additionalUnits, boolean
 			additionalUnits = new SourceFile[toAdd];
 		else
 			System.arraycopy(additionalUnits, 0, additionalUnits = new SourceFile[length + toAdd], 0, length);
+		Iterator<SourceFile> iterator = this.problemSourceFiles.iterator();
 		for (int i = 0; i < toAdd; i++)
-			additionalUnits[length + i] = (SourceFile) this.problemSourceFiles.get(i);
+			additionalUnits[length + i] = iterator.next();
 	}
 	String[] initialTypeNames = new String[units.length];
-	for (int i = 0, l = units.length; i < l; i++)
-		initialTypeNames[i] = units[i].initialTypeName;
+	for (int i = 0, l = units.length; i < l; i++) {
+		char[] moduleName = units[i].getModuleName();
+		initialTypeNames[i] = (moduleName == null)
+				? units[i].initialTypeName
+				: new StringBuilder(60).append(moduleName).append(':').append(units[i].initialTypeName).toString();
+	}
 	this.nameEnvironment.setNames(initialTypeNames, additionalUnits);
 	this.notifier.checkCancel();
 	try {
@@ -413,7 +429,7 @@ protected void createProblemFor(IResource resource, IMember javaElement, String
 		int end = range == null ? 1 : start + range.getLength();
 		marker.setAttributes(
 			new String[] {IMarker.MESSAGE, IMarker.SEVERITY, IMarker.CHAR_START, IMarker.CHAR_END, IMarker.SOURCE_ID},
-			new Object[] {message, new Integer(severity), new Integer(start), new Integer(end), JavaBuilder.SOURCE_ID});
+			new Object[] {message, Integer.valueOf(severity), Integer.valueOf(start), Integer.valueOf(end), JavaBuilder.SOURCE_ID});
 	} catch (CoreException e) {
 		throw internalException(e);
 	}
@@ -426,9 +442,8 @@ protected void deleteGeneratedFiles(IFile[] deletedGeneratedFiles) {
 protected SourceFile findSourceFile(IFile file, boolean mustExist) {
 	if (mustExist && !file.exists()) return null;
 
-	// assumes the file exists in at least one of the source folders & is not excluded
-	ClasspathMultiDirectory md = this.sourceLocations[0];
-	if (this.sourceLocations.length > 1) {
+	ClasspathMultiDirectory md = null;
+	if (this.sourceLocations.length > 0) {
 		IPath sourceFileFullPath = file.getFullPath();
 		for (int j = 0, m = this.sourceLocations.length; j < m; j++) {
 			if (this.sourceLocations[j].sourceFolder.getFullPath().isPrefixOf(sourceFileFullPath)) {
@@ -440,7 +455,7 @@ protected SourceFile findSourceFile(IFile file, boolean mustExist) {
 			}
 		}
 	}
-	return new SourceFile(file, md);
+	return md == null ? null: new SourceFile(file, md);
 }
 
 protected void finishedWith(String sourceLocator, CompilationResult result, char[] mainTypeName, ArrayList definedTypeNames, ArrayList duplicateTypeNames) {
@@ -474,11 +489,7 @@ protected IContainer createFolder(IPath packagePath, IContainer outputFolder) th
 	return folder;
 }
 
-
-
-/* (non-Javadoc)
- * @see org.aspectj.org.eclipse.jdt.internal.core.builder.ICompilationUnitLocator#fromIFile(org.eclipse.core.resources.IFile)
- */
+@Override
 public ICompilationUnit fromIFile(IFile file) {
 	return findSourceFile(file, true);
 }
@@ -486,7 +497,7 @@ public ICompilationUnit fromIFile(IFile file) {
 protected void initializeAnnotationProcessorManager(Compiler newCompiler) {
 	AbstractAnnotationProcessorManager annotationManager = JavaModelManager.getJavaModelManager().createAnnotationProcessorManager();
 	if (annotationManager != null) {
-		annotationManager.configureFromPlatform(newCompiler, this, this.javaBuilder.javaProject);
+		annotationManager.configureFromPlatform(newCompiler, this, this.javaBuilder.javaProject, this.compilationGroup == CompilationGroup.TEST);
 		annotationManager.setErr(new PrintWriter(System.err));
 		annotationManager.setOut(new PrintWriter(System.out));
 	}
@@ -559,7 +570,7 @@ protected Compiler newCompiler() {
 protected CompilationParticipantResult[] notifyParticipants(SourceFile[] unitsAboutToCompile) {
 	CompilationParticipantResult[] results = new CompilationParticipantResult[unitsAboutToCompile.length];
 	for (int i = unitsAboutToCompile.length; --i >= 0;)
-		results[i] = new CompilationParticipantResult(unitsAboutToCompile[i]);
+		results[i] = new CompilationParticipantResult(unitsAboutToCompile[i], this.compilationGroup == CompilationGroup.TEST);
 
 	// TODO (kent) do we expect to have more than one participant?
 	// and if so should we pass the generated files from the each processor to the others to process?
@@ -589,7 +600,7 @@ protected CompilationParticipantResult[] notifyParticipants(SourceFile[] unitsAb
 						uniqueFiles.add(unitsAboutToCompile[f]);
 				}
 				if (uniqueFiles.addIfNotIncluded(sourceFile) == sourceFile) {
-					CompilationParticipantResult newResult = new CompilationParticipantResult(sourceFile);
+					CompilationParticipantResult newResult = new CompilationParticipantResult(sourceFile, this.compilationGroup == CompilationGroup.TEST);
 					// is there enough room to add all the addedGeneratedFiles.length ?
 					if (toAdd == null) {
 						toAdd = new CompilationParticipantResult[addedGeneratedFiles.length];
@@ -599,6 +610,7 @@ protected CompilationParticipantResult[] notifyParticipants(SourceFile[] unitsAb
 							System.arraycopy(toAdd, 0, toAdd = new CompilationParticipantResult[length + addedGeneratedFiles.length], 0, length);
 					}
 					toAdd[added++] = newResult;
+					this.workQueue.add(sourceFile);
 				}
 			}
 		}
@@ -620,9 +632,9 @@ protected void processAnnotations(CompilationParticipantResult[] results) {
 		hasAnnotationProcessor = this.javaBuilder.participants[i].isAnnotationProcessor();
 	if (!hasAnnotationProcessor) return;
 
-	boolean foundAnnotations = this.filesWithAnnotations != null && this.filesWithAnnotations.elementSize > 0;
+	boolean foundAnnotations = this.filesWithAnnotations != null && this.filesWithAnnotations.size() > 0;
 	for (int i = results.length; --i >= 0;)
-		results[i].reset(foundAnnotations && this.filesWithAnnotations.includes(results[i].sourceFile));
+		results[i].reset(foundAnnotations && this.filesWithAnnotations.contains(results[i].sourceFile));
 
 	// even if no files have annotations, must still tell every annotation processor in case the file used to have them
 	for (int i = 0, l = this.javaBuilder.participants.length; i < l; i++)
@@ -698,8 +710,8 @@ protected void storeProblemsFor(SourceFile sourceFile, CategorizedProblem[] prob
 				new String[] {IMarker.MESSAGE, IMarker.SEVERITY, IJavaModelMarker.CATEGORY_ID, IMarker.SOURCE_ID},
 				new Object[] {
 					Messages.bind(Messages.build_incompleteClassPath, missingClassfileName),
-					new Integer(isInvalidClasspathError ? IMarker.SEVERITY_ERROR : IMarker.SEVERITY_WARNING),
-					new Integer(CategorizedProblem.CAT_BUILDPATH),
+					Integer.valueOf(isInvalidClasspathError ? IMarker.SEVERITY_ERROR : IMarker.SEVERITY_WARNING),
+					Integer.valueOf(CategorizedProblem.CAT_BUILDPATH),
 					JavaBuilder.SOURCE_ID
 				}
 			);
@@ -763,13 +775,13 @@ protected void storeProblemsFor(SourceFile sourceFile, CategorizedProblem[] prob
 			// standard attributes
 			int index = 0;
 			allValues[index++] = problem.getMessage(); // message
-			allValues[index++] = problem.isError() ? S_ERROR : S_WARNING; // severity
-			allValues[index++] = new Integer(id); // ID
-			allValues[index++] = new Integer(problem.getSourceStart()); // start
-			allValues[index++] = new Integer(problem.getSourceEnd() + 1); // end
-			allValues[index++] = new Integer(problem.getSourceLineNumber()); // line
+			allValues[index++] = problem.isError() ? S_ERROR : problem.isWarning() ? S_WARNING : S_INFO; // severity
+			allValues[index++] = Integer.valueOf(id); // ID
+			allValues[index++] = Integer.valueOf(problem.getSourceStart()); // start
+			allValues[index++] = Integer.valueOf(problem.getSourceEnd() + 1); // end
+			allValues[index++] = Integer.valueOf(problem.getSourceLineNumber()); // line
 			allValues[index++] = Util.getProblemArgumentsForMarker(problem.getArguments()); // arguments
-			allValues[index++] = new Integer(problem.getCategoryID()); // category ID
+			allValues[index++] = Integer.valueOf(problem.getCategoryID()); // category ID
 			// SOURCE_ID attribute for JDT problems
 			if (managedLength > 0)
 				allValues[index++] = JavaBuilder.SOURCE_ID;
@@ -815,10 +827,10 @@ protected void storeTasksFor(SourceFile sourceFile, CategorizedProblem[] tasks)
 			int index = 0;
 			allValues[index++] = task.getMessage();
 			allValues[index++] = priority;
-			allValues[index++] = new Integer(task.getID());
-			allValues[index++] = new Integer(task.getSourceStart());
-			allValues[index++] = new Integer(task.getSourceEnd() + 1);
-			allValues[index++] = new Integer(task.getSourceLineNumber());
+			allValues[index++] = Integer.valueOf(task.getID());
+			allValues[index++] = Integer.valueOf(task.getSourceStart());
+			allValues[index++] = Integer.valueOf(task.getSourceEnd() + 1);
+			allValues[index++] = Integer.valueOf(task.getSourceLineNumber());
 			allValues[index++] = Boolean.FALSE;
 			allValues[index++] = JavaBuilder.SOURCE_ID;
 			// optional extra attributes
