@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2014 IBM Corporation and others.
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -10,27 +10,51 @@
  *******************************************************************************/
 package org.aspectj.org.eclipse.jdt.internal.core.search.indexing;
 
-import java.io.*;
+import java.io.BufferedWriter;
+import java.io.File;
+import java.io.FileWriter;
+import java.io.IOException;
 import java.net.URL;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Locale;
+import java.util.Map;
 import java.util.zip.CRC32;
 
-import org.eclipse.core.resources.*;
+import org.eclipse.core.resources.IContainer;
+import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IFolder;
+import org.eclipse.core.resources.IProject;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.OperationCanceledException;
 import org.eclipse.core.runtime.Path;
-import org.aspectj.org.eclipse.jdt.core.*;
+import org.eclipse.core.runtime.SubMonitor;
+import org.aspectj.org.eclipse.jdt.core.IClasspathEntry;
+import org.aspectj.org.eclipse.jdt.core.IJavaProject;
+import org.aspectj.org.eclipse.jdt.core.JavaCore;
+import org.aspectj.org.eclipse.jdt.core.JavaModelException;
 import org.aspectj.org.eclipse.jdt.core.compiler.CharOperation;
-import org.aspectj.org.eclipse.jdt.core.search.*;
+import org.aspectj.org.eclipse.jdt.core.search.IJavaSearchScope;
+import org.aspectj.org.eclipse.jdt.core.search.SearchDocument;
+import org.aspectj.org.eclipse.jdt.core.search.SearchEngine;
+import org.aspectj.org.eclipse.jdt.core.search.SearchParticipant;
 import org.aspectj.org.eclipse.jdt.internal.compiler.ISourceElementRequestor;
 import org.aspectj.org.eclipse.jdt.internal.compiler.SourceElementParser;
 import org.aspectj.org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
 import org.aspectj.org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
+import org.aspectj.org.eclipse.jdt.internal.compiler.util.JRTUtil;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleLookupTable;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleSet;
-import org.aspectj.org.eclipse.jdt.internal.core.*;
-import org.aspectj.org.eclipse.jdt.internal.core.index.*;
+import org.aspectj.org.eclipse.jdt.internal.core.ClasspathEntry;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaModel;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaModelManager;
+import org.aspectj.org.eclipse.jdt.internal.core.JavaProject;
+import org.aspectj.org.eclipse.jdt.internal.core.index.DiskIndex;
+import org.aspectj.org.eclipse.jdt.internal.core.index.FileIndexLocation;
+import org.aspectj.org.eclipse.jdt.internal.core.index.Index;
+import org.aspectj.org.eclipse.jdt.internal.core.index.IndexLocation;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.indexer.Indexer;
+import org.aspectj.org.eclipse.jdt.internal.core.nd.java.JavaIndex;
 import org.aspectj.org.eclipse.jdt.internal.core.search.BasicSearchEngine;
 import org.aspectj.org.eclipse.jdt.internal.core.search.PatternSearchJob;
 import org.aspectj.org.eclipse.jdt.internal.core.search.processing.IJob;
@@ -47,6 +71,11 @@ public class IndexManager extends JobManager implements IIndexConstants {
 	// key = indexLocation path, value = an index
 	private SimpleLookupTable indexes = new SimpleLookupTable();
 
+	/**
+	 * The new indexer
+	 */
+	private Indexer indexer = Indexer.getInstance();
+
 	/* need to save ? */
 	private boolean needToSave = false;
 	private IPath javaPluginLocation = null;
@@ -55,15 +84,16 @@ public class IndexManager extends JobManager implements IIndexConstants {
 	// key = indexLocation path, value = index state integer
 	private SimpleLookupTable indexStates = null;
 	private File indexNamesMapFile = new File(getSavedIndexesDirectory(), "indexNamesMap.txt"); //$NON-NLS-1$
-	private File savedIndexNamesFile = new File(getSavedIndexesDirectory(), "savedIndexNames.txt"); //$NON-NLS-1$
 	private File participantIndexNamesFile = new File(getSavedIndexesDirectory(), "participantsIndexNames.txt"); //$NON-NLS-1$
 	private boolean javaLikeNamesChanged = true;
-	public static final Integer SAVED_STATE = new Integer(0);
-	public static final Integer UPDATING_STATE = new Integer(1);
-	public static final Integer UNKNOWN_STATE = new Integer(2);
-	public static final Integer REBUILDING_STATE = new Integer(3);
-	public static final Integer REUSE_STATE = new Integer(4);
-	
+	public static final Integer SAVED_STATE = 0;
+	public static final Integer UPDATING_STATE = 1;
+	public static final Integer UNKNOWN_STATE = 2;
+	public static final Integer REBUILDING_STATE = 3;
+	public static final Integer REUSE_STATE = 4;
+
+	private final IndexNamesRegistry nameRegistry = new IndexNamesRegistry(new File(getSavedIndexesDirectory(), 
+			"savedIndexNames.txt"), getJavaPluginWorkingLocation()); //$NON-NLS-1$
 	// search participants who register indexes with the index manager
 	private SimpleLookupTable participantsContainers = null;
 	private boolean participantUpdated = false;
@@ -141,7 +171,7 @@ public void cleanUpIndexes() {
 		if (count > 0)
 			removeIndexesState(locations);
 	}
-	deleteIndexFiles(knownPaths);
+	deleteIndexFiles(knownPaths, null);
 }
 /**
  * Compute the pre-built index location for a specified URL
@@ -189,17 +219,25 @@ public synchronized IndexLocation computeIndexLocation(IPath containerPath) {
 	}
 	return indexLocation;
 }
-public void deleteIndexFiles() {
-	if (DEBUG)
-		Util.verbose("Deleting index files"); //$NON-NLS-1$
-	this.savedIndexNamesFile.delete(); // forget saved indexes & delete each index file
+/**
+ * Use {@link #deleteIndexFiles(IProgressMonitor)}
+ */
+public final void deleteIndexFiles() {
 	deleteIndexFiles(null);
 }
-private void deleteIndexFiles(SimpleSet pathsToKeep) {
+public void deleteIndexFiles(IProgressMonitor monitor) {
+	if (DEBUG)
+		Util.verbose("Deleting index files"); //$NON-NLS-1$
+	this.nameRegistry.delete(); // forget saved indexes & delete each index file
+	deleteIndexFiles(null, monitor);
+}
+private void deleteIndexFiles(SimpleSet pathsToKeep, IProgressMonitor monitor) {
 	File[] indexesFiles = getSavedIndexesDirectory().listFiles();
 	if (indexesFiles == null) return;
 
+	SubMonitor subMonitor = SubMonitor.convert(monitor, indexesFiles.length);
 	for (int i = 0, l = indexesFiles.length; i < l; i++) {
+		subMonitor.split(1);
 		String fileName = indexesFiles[i].getAbsolutePath();
 		if (pathsToKeep != null && pathsToKeep.includes(new FileIndexLocation(indexesFiles[i]))) continue;
 		String suffix = ".index"; //$NON-NLS-1$
@@ -214,6 +252,7 @@ private void deleteIndexFiles(SimpleSet pathsToKeep) {
  * Creates an empty index at the given location, for the given container path, if none exist.
  */
 public synchronized void ensureIndexExists(IndexLocation indexLocation, IPath containerPath) {
+	this.indexer.makeWorkspacePathDirty(containerPath);
 	SimpleLookupTable states = getIndexStates();
 	Object state = states.get(indexLocation);
 	if (state == null) {
@@ -419,7 +458,7 @@ private SimpleLookupTable getIndexStates() {
 
 	this.indexStates = new SimpleLookupTable();
 	File indexesDirectoryPath = getSavedIndexesDirectory();
-	char[][] savedNames = readIndexState(getJavaPluginWorkingLocation().toOSString());
+	char[][] savedNames = this.nameRegistry.read(null);
 	if (savedNames != null) {
 		for (int i = 1, l = savedNames.length; i < l; i++) { // first name is saved signature, see readIndexState()
 			char[] savedName = savedNames[i];
@@ -525,6 +564,7 @@ public void indexResolvedDocument(SearchDocument searchDocument, SearchParticipa
  * Note: the actual operation is performed in background
  */
 public void indexAll(IProject project) {
+	this.indexer.makeDirty(project);
 	if (JavaCore.getPlugin() == null) return;
 
 	// Also request indexing of binaries on the classpath
@@ -553,11 +593,20 @@ public void indexLibrary(IPath path, IProject requestingProject, URL indexURL) {
 	this.indexLibrary(path, requestingProject, indexURL, false);
 }
 
+private IndexRequest getRequest(Object target, IPath jPath, IndexLocation indexFile, IndexManager manager, boolean updateIndex) {
+	return isJrt(((File) target).getName()) ? new AddJrtToIndex(jPath, indexFile, this, updateIndex) :
+		new AddJarFileToIndex(jPath, indexFile, this, updateIndex);
+}
+
+private boolean isJrt(String fileName) {
+	return fileName != null && fileName.endsWith(JRTUtil.JRT_FS_JAR);
+}
 /**
  * Trigger addition of a library to an index
  * Note: the actual operation is performed in background
  */
 public void indexLibrary(IPath path, IProject requestingProject, URL indexURL, final boolean updateIndex) {
+	this.indexer.makeWorkspacePathDirty(path);
 	// requestingProject is no longer used to cancel jobs but leave it here just in case
 	IndexLocation indexFile = null;
 	boolean forceIndexUpdate = false;
@@ -579,9 +628,11 @@ public void indexLibrary(IPath path, IProject requestingProject, URL indexURL, f
 	IndexRequest request = null;
 	Object target = JavaModel.getTarget(path, true);
 	if (target instanceof IFile) {
-		request = new AddJarFileToIndex((IFile) target, indexFile, this, forceIndexUpdate);
+		request = isJrt(((IFile) target).getFullPath().toOSString()) ? 
+				new AddJrtToIndex((IFile) target, indexFile, this, forceIndexUpdate) :
+					new AddJarFileToIndex((IFile) target, indexFile, this, forceIndexUpdate);
 	} else if (target instanceof File) {
-		request = new AddJarFileToIndex(path, indexFile, this, forceIndexUpdate);
+		request = getRequest(target, path, indexFile, this, forceIndexUpdate);
 	} else if (target instanceof IContainer) {
 		request = new IndexBinaryFolder((IContainer) target, this);
 	} else {
@@ -611,6 +662,7 @@ synchronized boolean addIndex(IPath containerPath, IndexLocation indexFile) {
  */
 public void indexSourceFolder(JavaProject javaProject, IPath sourceFolder, char[][] inclusionPatterns, char[][] exclusionPatterns) {
 	IProject project = javaProject.getProject();
+	this.indexer.makeWorkspacePathDirty(sourceFolder);
 	if (this.jobEnd > this.jobStart) {
 		// skip it if a job to index the project is already in the queue
 		IndexRequest request = new IndexAllProject(project, this);
@@ -632,6 +684,7 @@ public synchronized void jobWasCancelled(IPath containerPath) {
  * Advance to the next available job, once the current one has been completed.
  * Note: clients awaiting until the job count is zero are still waiting at this point.
  */
+@Override
 protected synchronized void moveToNextJob() {
 	// remember that one job was executed, and we will need to save indexes at some point
 	this.needToSave = true;
@@ -640,12 +693,14 @@ protected synchronized void moveToNextJob() {
 /**
  * No more job awaiting.
  */
+@Override
 protected void notifyIdle(long idlingTime){
 	if (idlingTime > 1000 && this.needToSave) saveIndexes();
 }
 /**
  * Name of the background process
  */
+@Override
 public String processName(){
 	return Messages.process_name;
 }
@@ -670,6 +725,7 @@ private void rebuildIndex(IndexLocation indexLocation, IPath containerPath) {
 	rebuildIndex(indexLocation, containerPath, false);
 }
 private void rebuildIndex(IndexLocation indexLocation, IPath containerPath, final boolean updateIndex) {
+	this.indexer.makeWorkspacePathDirty(containerPath);
 	Object target = JavaModel.getTarget(containerPath, true);
 	if (target == null) return;
 
@@ -685,9 +741,11 @@ private void rebuildIndex(IndexLocation indexLocation, IPath containerPath, fina
 	} else if (target instanceof IFolder) {
 		request = new IndexBinaryFolder((IFolder) target, this);
 	} else if (target instanceof IFile) {
-		request = new AddJarFileToIndex((IFile) target, null, this, updateIndex);
+		request = isJrt(((IFile) target).getFullPath().toOSString()) ? 
+				new AddJrtToIndex((IFile) target, null, this, updateIndex) :
+					new AddJarFileToIndex((IFile) target, null, this, updateIndex);
 	} else if (target instanceof File) {
-		request = new AddJarFileToIndex(containerPath, null, this, updateIndex);
+		request = getRequest(target, containerPath, null, this, updateIndex);
 	}
 	if (request != null)
 		request(request);
@@ -726,6 +784,7 @@ public synchronized Index recreateIndex(IPath containerPath) {
  * Note: the actual operation is performed in background
  */
 public void remove(String containerRelativePath, IPath indexedContainer){
+	this.indexer.makeWorkspacePathDirty(indexedContainer);
 	request(new RemoveFromIndex(containerRelativePath, indexedContainer, this));
 }
 /**
@@ -735,6 +794,7 @@ public void remove(String containerRelativePath, IPath indexedContainer){
 public synchronized void removeIndex(IPath containerPath) {
 	if (VERBOSE || DEBUG)
 		Util.verbose("removing index " + containerPath); //$NON-NLS-1$
+	this.indexer.makeWorkspacePathDirty(containerPath);
 	IndexLocation indexLocation = computeIndexLocation(containerPath);
 	Index index = getIndex(indexLocation);
 	File indexFile = null;
@@ -764,6 +824,7 @@ public synchronized void removeIndex(IPath containerPath) {
 public synchronized void removeIndexPath(IPath path) {
 	if (VERBOSE || DEBUG)
 		Util.verbose("removing index path " + path); //$NON-NLS-1$
+	this.indexer.makeWorkspacePathDirty(path);
 	Object[] keyTable = this.indexes.keyTable;
 	Object[] valueTable = this.indexes.valueTable;
 	IndexLocation[] locations = null;
@@ -810,6 +871,7 @@ public synchronized void removeIndexPath(IPath path) {
  * Removes all indexes whose paths start with (or are equal to) the given path.
  */
 public synchronized void removeIndexFamily(IPath path) {
+	this.indexer.makeWorkspacePathDirty(path);
 	// only finds cached index files... shutdown removes all non-cached index files
 	ArrayList toRemove = null;
 	Object[] containerPaths = this.indexLocations.keyTable;
@@ -830,6 +892,7 @@ public synchronized void removeIndexFamily(IPath path) {
  * Remove the content of the given source folder from the index.
  */
 public void removeSourceFolderFromIndex(JavaProject javaProject, IPath sourceFolder, char[][] inclusionPatterns, char[][] exclusionPatterns) {
+	this.indexer.makeWorkspacePathDirty(sourceFolder);
 	IProject project = javaProject.getProject();
 	if (this.jobEnd > this.jobStart) {
 		// skip it if a job to index the project is already in the queue
@@ -842,14 +905,17 @@ public void removeSourceFolderFromIndex(JavaProject javaProject, IPath sourceFol
 /**
  * Flush current state
  */
-public synchronized void reset() {
+@Override
+public void reset() {
 	super.reset();
-	if (this.indexes != null) {
-		this.indexes = new SimpleLookupTable();
-		this.indexStates = null;
+	synchronized (this) {
+		if (this.indexes != null) {
+			this.indexes = new SimpleLookupTable();
+			this.indexStates = null;
+		}
+		this.indexLocations = new SimpleLookupTable();
+		this.javaPluginLocation = null;
 	}
-	this.indexLocations = new SimpleLookupTable();
-	this.javaPluginLocation = null;
 }
 /**
  * Resets the index for a given path.
@@ -969,7 +1035,10 @@ public void saveIndexes() {
 	this.needToSave = !allSaved;
 }
 public void scheduleDocumentIndexing(final SearchDocument searchDocument, IPath container, final IndexLocation indexLocation, final SearchParticipant searchParticipant) {
+	IPath targetLocation = JavaIndex.getLocationForPath(new Path(searchDocument.getPath()));
+	this.indexer.makeDirty(targetLocation);
 	request(new IndexRequest(container, this) {
+		@Override
 		public boolean execute(IProgressMonitor progressMonitor) {
 			if (this.isCancelled || progressMonitor != null && progressMonitor.isCanceled()) return true;
 
@@ -990,12 +1059,18 @@ public void scheduleDocumentIndexing(final SearchDocument searchDocument, IPath
 			}
 			return true;
 		}
+		@Override
 		public String toString() {
 			return "indexing " + searchDocument.getPath(); //$NON-NLS-1$
 		}
+		@Override
+		public boolean waitNeeded() {
+			return false;
+		}
 	});
 }
 
+@Override
 public String toString() {
 	StringBuffer buffer = new StringBuffer(10);
 	buffer.append(super.toString());
@@ -1032,24 +1107,6 @@ private void readIndexMap() {
 	}
 	return;
 }
-private char[][] readIndexState(String dirOSString) {
-	try {
-		char[] savedIndexNames = org.aspectj.org.eclipse.jdt.internal.compiler.util.Util.getFileCharContent(this.savedIndexNamesFile, null);
-		if (savedIndexNames.length > 0) {
-			char[][] names = CharOperation.splitOn('\n', savedIndexNames);
-			if (names.length > 1) {
-				// First line is DiskIndex signature + saved plugin working location (see writeSavedIndexNamesFile())
-				String savedSignature = DiskIndex.SIGNATURE + "+" + dirOSString; //$NON-NLS-1$
-				if (savedSignature.equals(new String(names[0])))
-					return names;
-			}
-		}
-	} catch (IOException ignored) {
-		if (VERBOSE)
-			Util.verbose("Failed to read saved index file names"); //$NON-NLS-1$
-	}
-	return null;
-}
 private void readParticipantsIndexNamesFile() {
 	SimpleLookupTable containers = new SimpleLookupTable(3);
 	try {
@@ -1229,33 +1286,23 @@ private void writeParticipantsIndexNamesFile() {
 	}
 }
 private void writeSavedIndexNamesFile() {
-	BufferedWriter writer = null;
-	try {
-		writer = new BufferedWriter(new FileWriter(this.savedIndexNamesFile));
-		writer.write(DiskIndex.SIGNATURE);
-		writer.write('+');
-		writer.write(getJavaPluginWorkingLocation().toOSString());
-		writer.write('\n');
-		Object[] keys = this.indexStates.keyTable;
-		Object[] states = this.indexStates.valueTable;
-		for (int i = 0, l = states.length; i < l; i++) {
-			IndexLocation key = (IndexLocation) keys[i];
-			if (key != null && states[i] == SAVED_STATE) {
-				writer.write(key.fileName());
-				writer.write('\n');
-			}
-		}
-	} catch (IOException ignored) {
-		if (VERBOSE)
-			Util.verbose("Failed to write saved index file names", System.err); //$NON-NLS-1$
-	} finally {
-		if (writer != null) {
-			try {
-				writer.close();
-			} catch (IOException e) {
-				// ignore
-			}
+	Object[] keys = this.indexStates.keyTable;
+	Object[] states = this.indexStates.valueTable;
+	int numToSave = 0;
+	for (int i = 0, l = states.length; i < l; i++) {
+		IndexLocation key = (IndexLocation) keys[i];
+		if (key != null && states[i] == SAVED_STATE) {
+			numToSave++;
 		}
 	}
+	char[][] arrays = new char[numToSave][];
+	int idx = 0;
+	for (int i = 0, l = states.length; i < l; i++) {
+		IndexLocation key = (IndexLocation) keys[i];
+		if (key != null && states[i] == SAVED_STATE) {
+			arrays[idx++] = key.fileName().toCharArray();
+		}
+	}
+	this.nameRegistry.write(arrays);
 }
 }
