@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2015 IBM Corporation and others.
+ * Copyright (c) 2000, 2016 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -25,6 +25,7 @@ import org.aspectj.org.eclipse.jdt.internal.compiler.ast.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.env.IBinaryType;
 import org.aspectj.org.eclipse.jdt.internal.compiler.lookup.*;
 import org.aspectj.org.eclipse.jdt.internal.compiler.util.SimpleSet;
+import org.aspectj.org.eclipse.jdt.internal.core.BinaryMethod;
 import org.aspectj.org.eclipse.jdt.internal.core.search.BasicSearchEngine;
 
 @SuppressWarnings({ "rawtypes", "unchecked" })
@@ -57,9 +58,11 @@ public MethodLocator(MethodPattern pattern) {
 /*
  * Clear caches
  */
+@Override
 protected void clear() {
 	this.methodDeclarationsWithInvalidParam = new HashMap();
 }
+@Override
 protected int fineGrain() {
 	return this.pattern.fineGrain;
 }
@@ -104,6 +107,7 @@ private MethodBinding getMethodBinding(ReferenceBinding type, char[] methodName,
 	return null;
 }
 
+@Override
 public void initializePolymorphicSearch(MatchLocator locator) {
 	long start = 0;
 	if (BasicSearchEngine.VERBOSE) {
@@ -150,6 +154,23 @@ protected boolean isVirtualInvoke(MethodBinding method, MessageSend messageSend)
 			&& !(method.isDefault() && this.pattern.focus != null 
 			&& !CharOperation.equals(this.pattern.declaringPackageName, method.declaringClass.qualifiedPackageName()));
 }
+protected ReferenceBinding checkMethodRef(MethodBinding method, ReferenceExpression referenceExpression) {
+	boolean result = (!method.isStatic() && !method.isPrivate()
+		&&	referenceExpression.isMethodReference()  
+		&& !(method.isDefault() && this.pattern.focus != null 
+		&& !CharOperation.equals(this.pattern.declaringPackageName, method.declaringClass.qualifiedPackageName())));
+	if (result) {
+		Expression lhs = referenceExpression.lhs;
+		if (lhs instanceof NameReference) {
+			Binding binding = ((NameReference) lhs).binding;
+			if (binding instanceof ReferenceBinding)
+				return (ReferenceBinding) binding;
+		}
+	}
+	
+	return null;
+}
+@Override
 public int match(ASTNode node, MatchingNodeSet nodeSet) {
 	int declarationsLevel = IMPOSSIBLE_MATCH;
 	if (this.pattern.findReferences) {
@@ -170,6 +191,7 @@ public int match(ASTNode node, MatchingNodeSet nodeSet) {
 	return nodeSet.addMatch(node, declarationsLevel);
 }
 
+@Override
 public int match(LambdaExpression node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findDeclarations) return IMPOSSIBLE_MATCH;
 	if (this.pattern.parameterSimpleNames != null && this.pattern.parameterSimpleNames.length != node.arguments().length) return IMPOSSIBLE_MATCH;
@@ -181,6 +203,7 @@ public int match(LambdaExpression node, MatchingNodeSet nodeSet) {
 //public int match(ConstructorDeclaration node, MatchingNodeSet nodeSet) - SKIP IT
 //public int match(Expression node, MatchingNodeSet nodeSet) - SKIP IT
 //public int match(FieldDeclaration node, MatchingNodeSet nodeSet) - SKIP IT
+@Override
 public int match(MethodDeclaration node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findDeclarations) return IMPOSSIBLE_MATCH;
 
@@ -220,6 +243,7 @@ public int match(MethodDeclaration node, MatchingNodeSet nodeSet) {
 	// Method declaration may match pattern
 	return nodeSet.addMatch(node, resolve ? POSSIBLE_MATCH : ACCURATE_MATCH);
 }
+@Override
 public int match(MemberValuePair node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findReferences) return IMPOSSIBLE_MATCH;
 
@@ -227,6 +251,7 @@ public int match(MemberValuePair node, MatchingNodeSet nodeSet) {
 
 	return nodeSet.addMatch(node, this.pattern.mustResolve ? POSSIBLE_MATCH : ACCURATE_MATCH);
 }
+@Override
 public int match(MessageSend node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findReferences) return IMPOSSIBLE_MATCH;
 
@@ -241,6 +266,7 @@ public int match(MessageSend node, MatchingNodeSet nodeSet) {
 	return nodeSet.addMatch(node, this.pattern.mustResolve ? POSSIBLE_MATCH : ACCURATE_MATCH);
 }
 
+@Override
 public int match(ReferenceExpression node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findReferences) return IMPOSSIBLE_MATCH;
 	if (!matchesName(this.pattern.selector, node.selector)) return IMPOSSIBLE_MATCH;
@@ -250,6 +276,7 @@ public int match(ReferenceExpression node, MatchingNodeSet nodeSet) {
 	return nodeSet.addMatch(node, this.pattern.mustResolve ? POSSIBLE_MATCH : ACCURATE_MATCH);
 }
 
+@Override
 public int match(Annotation node, MatchingNodeSet nodeSet) {
 	if (!this.pattern.findReferences) return IMPOSSIBLE_MATCH;
 	MemberValuePair[] pairs = node.memberValuePairs();
@@ -269,6 +296,7 @@ public int match(Annotation node, MatchingNodeSet nodeSet) {
 //public int match(TypeDeclaration node, MatchingNodeSet nodeSet) - SKIP IT
 //public int match(TypeReference node, MatchingNodeSet nodeSet) - SKIP IT
 
+@Override
 protected int matchContainer() {
 	if (this.pattern.findReferences) {
 		// need to look almost everywhere to find in javadocs and static import
@@ -280,6 +308,7 @@ protected int matchContainer() {
  * @see org.aspectj.org.eclipse.jdt.internal.core.search.matching.PatternLocator#matchLevelAndReportImportRef(org.aspectj.org.eclipse.jdt.internal.compiler.ast.ImportReference, org.aspectj.org.eclipse.jdt.internal.compiler.lookup.Binding, org.aspectj.org.eclipse.jdt.internal.core.search.matching.MatchLocator)
  * Accept to report match of static field on static import
  */
+@Override
 protected void matchLevelAndReportImportRef(ImportReference importRef, Binding binding, MatchLocator locator) throws CoreException {
 	if (importRef.isStatic() && binding instanceof MethodBinding) {
 		super.matchLevelAndReportImportRef(importRef, binding, locator);
@@ -314,6 +343,7 @@ protected int matchMethod(MethodBinding method, boolean skipImpossibleArg) {
 		boolean foundTypeVariable = false;
 		MethodBinding focusMethodBinding = null;
 		boolean checkedFocus = false;
+		boolean isBinary = this.pattern!= null && this.pattern.focus instanceof BinaryMethod;
 		// verify each parameter
 		for (int i = 0; i < parameterCount; i++) {
 			TypeBinding argType = method.parameters[i];
@@ -327,7 +357,8 @@ protected int matchMethod(MethodBinding method, boolean skipImpossibleArg) {
 				if (focusMethodBinding != null) {// textual comparison insufficient
 					TypeBinding[] parameters = focusMethodBinding.parameters;
 					if (parameters.length >= parameterCount) {
-						newLevel = argType.isEquivalentTo((parameters[i])) ? ACCURATE_MATCH : IMPOSSIBLE_MATCH;
+						newLevel = (isBinary ? argType.erasure().isEquivalentTo((parameters[i].erasure())) :argType.isEquivalentTo((parameters[i]))) ? 
+								ACCURATE_MATCH : IMPOSSIBLE_MATCH;
 						foundLevel = true;
 					}
 				}
@@ -420,12 +451,14 @@ private boolean matchOverriddenMethod(ReferenceBinding type, MethodBinding metho
 	}
 	return false;
 }
+@Override
 protected void matchReportReference(ASTNode reference, IJavaElement element, Binding elementBinding, int accuracy, MatchLocator locator) throws CoreException {
 	matchReportReference(reference, element, null, null, elementBinding, accuracy, locator);
 }
 /**
  * @see org.aspectj.org.eclipse.jdt.internal.core.search.matching.PatternLocator#matchReportReference(org.aspectj.org.eclipse.jdt.internal.compiler.ast.ASTNode, org.aspectj.org.eclipse.jdt.core.IJavaElement, Binding, int, org.aspectj.org.eclipse.jdt.internal.core.search.matching.MatchLocator)
  */
+@Override
 protected void matchReportReference(ASTNode reference, IJavaElement element, IJavaElement localElement, IJavaElement[] otherElements, Binding elementBinding, int accuracy, MatchLocator locator) throws CoreException {
 	MethodBinding methodBinding = (reference instanceof MessageSend) ? ((MessageSend)reference).binding: ((elementBinding instanceof MethodBinding) ? (MethodBinding) elementBinding : null);
 	if (this.isDeclarationOfReferencedMethodsPattern) {
@@ -579,6 +612,7 @@ private boolean methodParametersEqualsPattern(MethodBinding method) {
 	}
 	return true;
 }
+@Override
 public SearchMatch newDeclarationMatch(ASTNode reference, IJavaElement element, Binding elementBinding, int accuracy, int length, MatchLocator locator) {
 	if (elementBinding != null) {
 		MethodBinding methodBinding = (MethodBinding) elementBinding;
@@ -613,6 +647,7 @@ public SearchMatch newDeclarationMatch(ASTNode reference, IJavaElement element,
 	}
 	return super.newDeclarationMatch(reference, element, elementBinding, accuracy, length, locator);
 }
+@Override
 protected int referenceType() {
 	return IJavaElement.METHOD;
 }
@@ -673,6 +708,7 @@ protected void reportDeclaration(MethodBinding methodBinding, MatchLocator locat
 		}
 	}
 }
+@Override
 public int resolveLevel(ASTNode possibleMatchingNode) {
 	if (this.pattern.findReferences) {
 		if (possibleMatchingNode instanceof MessageSend) {
@@ -700,6 +736,7 @@ public int resolveLevel(ASTNode possibleMatchingNode) {
 	}
 	return IMPOSSIBLE_MATCH;
 }
+@Override
 public int resolveLevel(Binding binding) {
 	if (binding == null) return INACCURATE_MATCH;
 	if (!(binding instanceof MethodBinding)) return IMPOSSIBLE_MATCH;
@@ -793,7 +830,28 @@ protected int resolveLevel(ReferenceExpression referenceExpression) {
 
 	// receiver type
 	if (this.pattern.declaringSimpleName == null && this.pattern.declaringQualification == null) return methodLevel; // since any declaring class will do
-	int declaringLevel = resolveLevelForType(this.pattern.declaringSimpleName, this.pattern.declaringQualification, method.declaringClass);
+	int declaringLevel;
+	ReferenceBinding ref = checkMethodRef(method, referenceExpression);
+	if (ref != null) {
+		declaringLevel = resolveLevelAsSubtype(this.pattern.declaringSimpleName, this.pattern.declaringQualification, ref, method.selector, method.parameters, ref.qualifiedPackageName(), method.isDefault());
+		if (declaringLevel == IMPOSSIBLE_MATCH) {
+			if (method.declaringClass == null || this.allSuperDeclaringTypeNames == null) {
+				declaringLevel = INACCURATE_MATCH;
+			} else {
+				char[][][] superTypeNames = (method.isDefault() && this.pattern.focus == null) ? this.samePkgSuperDeclaringTypeNames: this.allSuperDeclaringTypeNames;
+				if (superTypeNames != null && resolveLevelAsSuperInvocation(ref, method.parameters, superTypeNames, true)) {
+						declaringLevel = methodLevel // since this is an ACCURATE_MATCH so return the possibly weaker match
+							| SUPER_INVOCATION_FLAVOR; // TODO: not an invocation really but ref -> add flavor to returned level
+				}
+			}
+		}
+		if ((declaringLevel & FLAVORS_MASK) != 0) {
+			// level got some flavors => return it
+			return declaringLevel;
+		}
+	} else {
+		declaringLevel = resolveLevelForType(this.pattern.declaringSimpleName, this.pattern.declaringQualification, method.declaringClass);
+	}
 	return (methodLevel & MATCH_LEVEL_MASK) > (declaringLevel & MATCH_LEVEL_MASK) ? declaringLevel : methodLevel; // return the weaker match
 }
 
@@ -902,6 +960,7 @@ private boolean resolveLevelAsSuperInvocation(ReferenceBinding type, TypeBinding
 	}
 	return false;
 }
+@Override
 public String toString() {
 	return "Locator for " + this.pattern.toString(); //$NON-NLS-1$
 }
